/******************************************************************************
 **
 ** $Id: misc.h 180 2009-04-19 00:33:18Z AndrewLahiff $
 **
 ******************************************************************************
 ** 
 ** Copyright (C) 2007-2009 Andrew Lahiff.
 ** 
 ** This file is part of plotcf.
 **
 ** plotcf is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 ** 
 ** plotcf is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 ** 
 ** You should have received a copy of the GNU General Public License
 ** along with plotcf.  If not, see <http://www.gnu.org/licenses/>.
 ** 
 *******************************************************************************/

#ifndef MISC_H_
#define MISC_H_

#include <cmath>

#include <QString>
#include <QDate>
#include <QRegExp>

/*******************************************************************************
 **
 ** convert time in HH:MM:SS format to seconds
 **
 *******************************************************************************/

double time_to_tis(char timein[]);

/*******************************************************************************
 **
 ** convert a date in QString YYYY-MM-DD format to QDate
 **
 *******************************************************************************/

QDate toQDate(QString in);

/*******************************************************************************
 **
 ** add seconds to a time stamp
 **
 *******************************************************************************/

QString timeBounds(QString timeStamp, double offset);

/*******************************************************************************
 **
 ** returns 'true' if DATASET_ID corresponds to something that should be
 ** plotted as a spectrogram. Most instruments haven't bothered to use
 ** DISPLAYTYPE unfortunately.
 **
 *******************************************************************************/

bool isSpectro(QString in);

/*******************************************************************************
 **
 ** return the maximum value in an array of a specified size
 **
 *******************************************************************************/

int maxValueOf(int array[], int size);

/*******************************************************************************
 **
 ** is a floating point number a fill value?
 **
 *******************************************************************************/

bool fl_compare(double a, double b);

/*******************************************************************************
 **
 ** print help message
 **
 *******************************************************************************/

void printHelp();

/*******************************************************************************
 **
 ** find appropriate S/C potential value
 **
 *******************************************************************************/

void getPotentialValue(float, float, int *, int, double *, double *, float *);

class ReadFile; // forward declaration

QString getVarNameEnergy(ReadFile& file, int varNum);
QString getVarNamePolar(ReadFile& file, int varNum);
QString getVarNameAzimuth(ReadFile& file, int varNum);
int varNumFromName(ReadFile& file, QString name);

#endif // MISC_H_
