/******************************************************************************
 **
 ** $Id: plotcf.h 184 2009-11-01 07:46:35Z AndrewLahiff $
 **
 ******************************************************************************
 ** 
 ** Copyright (C) 2007-2009 Andrew Lahiff.
 ** 
 ** This file is part of plotcf.
 **
 ** plotcf is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 ** 
 ** plotcf is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 ** 
 ** You should have received a copy of the GNU General Public License
 ** along with plotcf.  If not, see <http://www.gnu.org/licenses/>.
 ** 
 *******************************************************************************/

#ifndef PLOTCF_H_
#define PLOTCF_H_

#include <QApplication>

#include <iostream>
#include <string>
#include <stdlib.h>
#include <vector>
#include <ctime>

#define VERSION "0.8.27"
#define PLOTCF_DATE "2010-02-18"

#define MAX_SIZE 60
#define MAX_FILES 10
#define MAX_SPECTRO 12
#define MAX_ARRAY_SIZE 86400
#define MAX_PLOTS_PER_PANEL 100

#define _ENERGY 2
#define _POLAR 1
#define _AZIMUTH 0

#define MY_FILL -123456789

#define SPECTROGRAM_PLOT 1
#define TIMESERIES_PLOT 2
#define SAUVAUD_PLOT 3

#define SPECTRO_ENERGY_TIME 0
#define SPECTRO_PA_TIME 1
#define SPECTRO_ENERGY_PA_TIME 2

#define PNG 0
#define JPG 1
#define TIFF 2
#define BMP 3
#define PS 4

#define MAX_DISTINCT_SPACECRAFT 10

/*******************************************************************************
 **
 ** structure for storing information about what will be in each plot panel
 **
 *******************************************************************************/

struct panelPlotStruct
{

// plot settings

   QString panelLabel, yLabel, cLabel;

   double xAxisMin, xAxisMax;
   double yAxisMin, yAxisMax;
   double cBarMin, cBarMax;
   
   int yAxisScaleType;	
    
   int yMajTicks, yMinTicks;
   int xMinTicks;
   double xMajTickDuration;

// time series data

   int numTimeSeries;
   int tsFile[MAX_PLOTS_PER_PANEL];           // source of data to plot - file
   int tsVar[MAX_PLOTS_PER_PANEL];            // source of data to plot - variable number
   int tsType[MAX_PLOTS_PER_PANEL];           // source of data to plot - data type 
   int tsVarComp[MAX_PLOTS_PER_PANEL];        // source of data to plot - component of vector
   int tsDataRef[MAX_PLOTS_PER_PANEL];        // offset in data storage array
   float tsFillValue[MAX_PLOTS_PER_PANEL];    // fill values
   QString legLabel[MAX_PLOTS_PER_PANEL];     // legend 
   int plotHow[MAX_PLOTS_PER_PANEL];          // line or symbol to use
   QString colour[MAX_PLOTS_PER_PANEL];       // line/point colour
   int tsFromDiff;                            // are the time series data in this panel from different sources?
   double multFactor[MAX_PLOTS_PER_PANEL];    // multiplication factors for each time series plot
   QString tsDATASET_ID[MAX_PLOTS_PER_PANEL]; // DATASET_ID
   
// spectrograms   
   
   int numSpectrograms;
   int spFile[MAX_PLOTS_PER_PANEL];             // source of data to plot - file
   int spVar[MAX_PLOTS_PER_PANEL];              // source of data to plot - variable number
   int spPolarComp;                             // polar bin to plot
   int spVars[MAX_PLOTS_PER_PANEL][3];          // variable containing azimuth, polar, energy
   int spDepends[MAX_PLOTS_PER_PANEL][3];       // which DEPEND_i contains azimuth, polar energy
   float spFillValue[MAX_PLOTS_PER_PANEL];      // fill values
   int spMaster;                                // this is the 1st polar bin of a series of polar bins
   int spSlave;                                 // this is > 1st polar bin of a series of polar bins
   QString spDATASET_ID[MAX_PLOTS_PER_PANEL];   // DATASET_ID
   int scN[MAX_PLOTS_PER_PANEL];                // spacecraft number
   double multFactorSp[MAX_PLOTS_PER_PANEL];    // multiplication factors for each spectrogram
   float multFactorSpY[MAX_PLOTS_PER_PANEL];    // y-axis multiplication factors for each spectrogram
   int spPotentialCorrect[MAX_PLOTS_PER_PANEL]; // apply spacecraft potential correction
};

/*******************************************************************************
 **
 ** structure for storing additional variable meta data
 **
 *******************************************************************************/

struct VarMetaData
{
   double SCALEMIN, SCALEMAX;
   QString DISPLAYTYPE, SCALETYP;
};

/*******************************************************************************
 **
 ** structure for storing plot numbers and heights
 **
 *******************************************************************************/

struct PlotHeightStruct
{
   int plot;
   double height;
};

#endif // PLOTCF_H_
