/*******************************************************************************
 **
 ** $Id: QWheelPlot.h 175 2009-03-28 11:50:15Z AndrewLahiff $
 **
 ****************************************************************************** 
 ** 
 ** Copyright (C) 2007-2009 Andrew Lahiff.
 ** 
 ** This file is part of plotcf.
 **
 ** plotcf is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 ** 
 ** plotcf is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 ** 
 ** You should have received a copy of the GNU General Public License
 ** along with plotcf.  If not, see <http://www.gnu.org/licenses/>.
 ** 
 *******************************************************************************/

#ifndef QWHEELPLOT_H_
#define QWHEELPLOT_H_

#include <QImage>
#include <QPainter>
#include <QFont>
#include <QColor>
#include <QWidget>
#include <QTextDocument>
#include <QSizeF>

#include <math.h>

#define QPGDEG2RAD  0.0174532925

#define LHS 0
#define RHS 1

#define ALIGN_LEFT 0
#define ALIGN_RIGHT 1
#define ALIGN_CENTRE 2

#define QW_PNG 0
#define QW_JPG 1
#define QW_TIFF 2
#define QW_BMP 3
#define QW_PS 4

class QWheelPlot
{
   public:
      QWheelPlot(int, int, int, int, double, double, double, double, int, bool = true);
      ~QWheelPlot();

      QImage *plot;
      int plotHeight;
      int plotWidth;
      double yAxisMin;
      double yAxisMax;
      double cAxisMin;
      double cAxisMax;
      int numR;
      int numT;

      float *rlo;
      float *rhi;
      float *thlo;
      float *thhi;
      float *z;

      void drawFrame(QString, QString, QString, QString, QString);
      void saveToFile(QString, QString = "");
      double SafeLog(double);
      void drawArc(float, float, float, float, double);
      void drawPlot();
      void setInvisibleValue(double);
      void drawSideTitle(int, QString);
      QColor colourBarDefinition(double);
      void setColourBarDefinition(int);

      void drawLabel(QString text);
      void dontShowLabel(bool show) { noShowLabel = show; }

   private:
      bool noShowLabel;
      QSizeF drawRichText(QString, QFont, int, QPainter *, int = 0);

      double cornerX;
      double cornerY;
      double myPlotHeight;
      double myPlotWidth;
      double xMarginL;
      double xMarginR;
      double yMargin;
      double gap;
      double colourBarWidth;
      double gapBetweenAxisAndTick;
      int iPlotTickMajor, iPlotTickMinor;
      double invisibleValue;

      QFont fontLabels, fontTicks, fontTitles, fontSubTitles, fontMarkers, fontPL;

      int colourBarDef;

      bool useGUI;

      double imageWidthPlot, imageHeightPlot;

};

#endif /*QWHEELPLOT_H_*/
