/*******************************************************************************
 **
 ** $Id: QsiPlot.C 175 2009-03-28 11:50:15Z AndrewLahiff $
 **
 ****************************************************************************** 
 ** 
 ** Copyright (C) 2007-2009 Andrew Lahiff.
 ** 
 ** This file is part of plotcf.
 **
 ** plotcf is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 ** 
 ** plotcf is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 ** 
 ** You should have received a copy of the GNU General Public License
 ** along with plotcf.  If not, see <http://www.gnu.org/licenses/>.
 ** 
 *******************************************************************************/

#include "QsiPlot.h"

#include <iostream>
using namespace std;

//-------------------------------------------------------------------------------------------------
//
// convert time in milliseconds to hours, mins, secs, milliseconds
//
//-------------------------------------------------------------------------------------------------

void QsiPlot::toTime(int milliseconds, int *hour, int *min, int *sec, int *msec)
{
    float V;
    short H, M, S;
    V = milliseconds / 3600000;
    H = (short) V;
    milliseconds -= 3600000 * H;
    V = milliseconds / 60000;
    M = (short) V;
    milliseconds -= 60000 * M;
    V = milliseconds / 1000;
    S = (short) V;
    milliseconds -= 1000 * S;

   *hour = H;
   *min = M;
   *sec = S;
   *msec = milliseconds;
}

//-----------------------------------------------------------------------
//
// toTime
// - convert time in milliseconds to time string
//    
//-----------------------------------------------------------------------
      
QString QsiPlot::toTimeString(int milliseconds, int format)
{     
    char str[20];
    float V;
    short H, M, S;
    V = milliseconds / 3600000;
    H = (short) V;
    milliseconds -= 3600000 * H;
    V = milliseconds / 60000;
    M = (short) V;
    milliseconds -= 60000 * M;
    V = milliseconds / 1000;
    S = (short) V;
    milliseconds -= 1000 * S;
    if (format == 0)
       sprintf (str, "%02d:%02d:%02d", H, M, S);
    else 
       sprintf (str, "%02d:%02d", H, M);
    return(QString(str));
}

//-------------------------------------------------------------------------------------------------
//
// QsiPlot constructor
//
//-------------------------------------------------------------------------------------------------

QsiPlot::QsiPlot(int imageWidth, int imageHeight, QString pageTitle, int defaultFontSize, bool gui)
{
   noShowLabel = false;
   pageTitle = pageTitle_;

   imageWidthPlot = imageWidth;
   imageHeightPlot = imageHeight;
   plot = new QImage(imageWidth, imageHeight, QImage::Format_RGB32);

   lMarginDef = 80;
   rMarginDef = 85;
   tMarginDef = 6;
   bMarginDef = 6;

   colourBarWidth = 18;

   for (int i=0; i<10; i++)
   {
      showValueX[i] = pTrue;
   }

   iPlotLineWidth = 2;
   iPlotTickMajor = 8;
   iPlotTickMinor = 4;
   iCurveLineWidth = 2;

   gapBetweenAxisAndTick = 3.0;

   heightOfValuesX = 18 + 10;

   titleHeight = 0;

   timeStringFormat = "hh:mm:ss";
   xAxisTimeString = 0;

   layoutDone = false;

// default fonts

   if (defaultFontSize < 1)
   {
#ifdef Q_WS_MAC
      defaultFontSize = 13;
#else
      defaultFontSize = 11;
#endif
   }

   QString defaultFontFamily = "Arial";

   fontTicks.setFamily(defaultFontFamily);
   fontTicks.setPointSize(defaultFontSize+1);
   fontTicksX.setFamily(defaultFontFamily);
   fontTicksX.setPointSize(defaultFontSize);
   fontTicksColour.setFamily(defaultFontFamily);
   fontTicksColour.setPointSize(defaultFontSize+1);
   fontLabels.setFamily(defaultFontFamily);
   fontLabels.setPointSize(defaultFontSize+2);
   fontTitle.setFamily(defaultFontFamily);
   fontTitle.setPointSize(defaultFontSize+2);   
//   fontTitle.setWeight(QFont::Bold);
   fontSubTitle.setFamily(defaultFontFamily);
   fontSubTitle.setPointSize(defaultFontSize+1); 
   fontPanelTitle.setFamily(defaultFontFamily);
   fontPanelTitle.setPointSize(defaultFontSize+1);
   fontPanelTitle.setWeight(QFont::Bold);
   fontLegends.setFamily(defaultFontFamily);
   fontLegends.setPointSize(defaultFontSize-1);
   fontColourBox.setFamily(defaultFontFamily);
   fontColourBox.setPointSize(defaultFontSize+1);
   fontSauv.setFamily(defaultFontFamily);
   fontSauv.setPointSize(defaultFontSize-2);
   fontPL.setFamily(defaultFontFamily);
   fontPL.setPointSize(defaultFontSize-4);

// symbols

   symbolToPlot = new QImage(symbolWidth, symbolHeight, QImage::Format_ARGB32);

// grid

   xGrid = false;
   yGrid = false;

// default colour bar definition

   colourBarDef = 0;   

// text type

   useGUI = gui;

// size scaling

   for (int i=0; i<20; i++) plotScaleFactorY[i] = 1.0;

}

//-------------------------------------------------------------------------------------------------
//
// destructor
//
//-------------------------------------------------------------------------------------------------

QsiPlot::~QsiPlot()
{
   delete plot;
   delete symbolToPlot;
}

//-------------------------------------------------------------------------------------------------
//
// set line width for plot frame and ticks
//
//-------------------------------------------------------------------------------------------------

void QsiPlot::setPlotLineWidth(int width)
{
   iPlotLineWidth = width;
}

//-------------------------------------------------------------------------------------------------
//
// set line width for curves
//
//-------------------------------------------------------------------------------------------------

void QsiPlot::setCurveLineWidth(int width)
{
   iCurveLineWidth = width;
}   

//-------------------------------------------------------------------------------------------------
//
// set background colour for entire page
//
//-------------------------------------------------------------------------------------------------

void QsiPlot::setBackgroundColor(QColor backgroundColor)
{
   plot->fill(backgroundColor.rgb());
}

//-------------------------------------------------------------------------------------------------
//
// set size
//
//-------------------------------------------------------------------------------------------------

void QsiPlot::setSize(int row, int col, double factor)
{
   col = col;
   plotScaleFactorY[row] = factor;
}

//-------------------------------------------------------------------------------------------------
//
// set multiple plot layout
//
//-------------------------------------------------------------------------------------------------

void QsiPlot::setLayout(int rows, int cols, QString title, QString subTitle)
{

   if (!title.isEmpty() && subTitle.isEmpty())
   {
      titleHeight = 20;
      QPainter painter(plot);
      painter.translate(QPointF(lMarginDef, (titleHeight + tMarginDef)/2.));
      drawRichText(title, fontTitle, ALIGN_LEFT, &painter);     
   }
   else if (!title.isEmpty() && !subTitle.isEmpty())
   {
      titleHeight = 40;
      QPainter painter(plot);
      painter.translate(QPointF(lMarginDef, (titleHeight + tMarginDef)/3.));
      drawRichText(title, fontTitle, ALIGN_LEFT, &painter);
      painter.resetMatrix();
      painter.translate(QPointF(lMarginDef, 2.0*(titleHeight + tMarginDef)/3.));
      drawRichText(subTitle, fontSubTitle, ALIGN_LEFT, &painter);     
   }

   numRows = rows;
   numCols = cols;
   numTotal = numRows*numCols;

// font scaling factor for y-axis ticks & labels; legends
// - helps to ensure text doesn't overlap other text or is too big when
//   there are many panels

   if (rows > 4)
   {
      double factor = 1.0 - ((double)rows)*0.015;
      double origFontSize = fontTicks.pointSizeF();
      fontTicks.setPointSizeF(origFontSize*factor);
      origFontSize = fontLabels.pointSizeF();
      fontLabels.setPointSizeF(origFontSize*factor);
      origFontSize = fontLegends.pointSizeF();
      fontLegends.setPointSizeF(origFontSize*factor);
   }

// for many panels, make the gap between panels smaller

   if (rows > 6)
   {
      tMarginDef = 3;
      bMarginDef = 3;
   }

}

//-------------------------------------------------------------------------------------------------
//
// set plot type
//
//-------------------------------------------------------------------------------------------------

void QsiPlot::setPlotType(int iRow, int iCol, int plotTypeUser)
{
   plotType[indexPosition(iRow, iCol)] = plotTypeUser;
}

//-------------------------------------------------------------------------------------------------
//
// set scale type for the Z-axis
//
//-------------------------------------------------------------------------------------------------

void QsiPlot::setScaleTypeZ(int iRow, int iCol, int scaleTypeUser)
{
   scaleTypeZ[indexPosition(iRow, iCol)] = scaleTypeUser;
}

//-------------------------------------------------------------------------------------------------
//
// set scale type for the Y-axis
//
//-------------------------------------------------------------------------------------------------

void QsiPlot::setScaleTypeY(int iRow, int iCol, int scaleTypeUser)
{
   scaleTypeY[indexPosition(iRow, iCol)] = scaleTypeUser;
}

//-------------------------------------------------------------------------------------------------
//
// set scale type for the X-axis
//
//-------------------------------------------------------------------------------------------------

void QsiPlot::setScaleTypeX(int iRow, int iCol, int scaleTypeUser)
{
   scaleTypeX[indexPosition(iRow, iCol)] = scaleTypeUser;
}

//-------------------------------------------------------------------------------------------------
//
// set grid
// 
//-------------------------------------------------------------------------------------------------

void QsiPlot::setGrid(bool userXgrid, bool userYgrid)
{
   xGrid = userXgrid;
   yGrid = userYgrid;
}

//-------------------------------------------------------------------------------------------------
//
// draw plot frame
//
//-------------------------------------------------------------------------------------------------

void QsiPlot::drawFrame(int iRow, int iCol, bool draw)
{

   int colMarginOffset = 15;

   int iFy = 0;
   for (int i=0; i<numRows; i++)
   {
      if (plotScaleFactorY[i] < 1.0) iFy++;
   }     

   double gap_spacing = 0.0;
   if (iFy > 0) gap_spacing = 10.0;
   
   if (!layoutDone)
   {
      layoutDone = true;
      int totalTitles = 0;
      for (int i=0; i<numRows; i++) // only need to know the number of titles per column
      {
         allValuesY[i] = totalTitles;
         if (showValueX[i] == pTrue) totalTitles++;
      }
      double titleOffset = (double)totalTitles*heightOfValuesX;
      widthSubPlot = (double)(imageWidthPlot - lMarginDef - rMarginDef)/(double)numCols;

      double r_numRows = (double)numRows;
      if (iFy > 0)
      {
         r_numRows = 1.0;
      }

      heightSubPlot = (double)(imageHeightPlot - titleHeight - titleOffset - 7. - gap_spacing*numRows)/r_numRows;
   }   

   QPen pen;
   pen.setWidth(iPlotLineWidth);

   double myPlotWidth = widthSubPlot - colMarginOffset;
   double myPlotHeight = (heightSubPlot - tMarginDef - bMarginDef)*plotScaleFactorY[iRow];
   double startX = (double)indexPositionReturn(iRow, iCol, QS_COL)*widthSubPlot + lMarginDef;
   double startY = (double)indexPositionReturn(iRow, iCol, QS_ROW)*heightSubPlot + tMarginDef + titleHeight;

   if (iFy > 0)
   {
      startY = tMarginDef + titleHeight;
      for (int i=0; i<iRow; i++)
      {
         if (plotScaleFactorY[i] < 1.0)
         {
            startY += plotScaleFactorY[i]*heightSubPlot + gap_spacing;
         }
      }
   }

   plotWidths[indexPosition(iRow, iCol)] = myPlotWidth;
   plotHeights[indexPosition(iRow, iCol)] = myPlotHeight;
   plotStartsX[indexPosition(iRow, iCol)] = startX;
   plotStartsY[indexPosition(iRow, iCol)] = startY + (double)allValuesY[indexPosition(iRow, iCol)]*heightOfValuesX;

   iPlotWidth = (int)myPlotWidth;
   iPlotHeight = (int)myPlotHeight;

   if (draw)
   {
      QPainter painter(plot);
      painter.setPen(pen);
      painter.drawRect(QRectF(plotStartsX[indexPosition(iRow, iCol)],
                              plotStartsY[indexPosition(iRow, iCol)], myPlotWidth, myPlotHeight));
   }
}

//-------------------------------------------------------------------------------------------------
//
// set plot axis limits
//
//-------------------------------------------------------------------------------------------------

void QsiPlot::setPlotAxisLimits(int iRow, int iCol, double xLow, double xUpp, double yLow, double yUpp)
{
   axisLimits[indexPosition(iRow, iCol)][0][0] = xLow;
   axisLimits[indexPosition(iRow, iCol)][0][1] = xUpp;
   axisLimits[indexPosition(iRow, iCol)][1][0] = yLow;
   axisLimits[indexPosition(iRow, iCol)][1][1] = yUpp;

   axisLimitsTicks[indexPosition(iRow, iCol)][0][0] = xLow;
   axisLimitsTicks[indexPosition(iRow, iCol)][0][1] = xUpp;
   axisLimitsTicks[indexPosition(iRow, iCol)][1][0] = yLow;
   axisLimitsTicks[indexPosition(iRow, iCol)][1][1] = yUpp;
}

//-------------------------------------------------------------------------------------------------
//
// draw horizontal line at a particular y-axis value
//
//-------------------------------------------------------------------------------------------------

void QsiPlot::drawLineY(int iRow, int iCol, QColor myColor, int plotLine, double yValue)
{
   double myPlotWidth = plotWidths[indexPosition(iRow, iCol)];
   double myPlotHeight = plotHeights[indexPosition(iRow, iCol)];
   double myStartX = plotStartsX[indexPosition(iRow, iCol)];
   double myStartY = plotStartsY[indexPosition(iRow, iCol)];

   QPainter painter(plot);
   QPen pen;
   pen.setWidth(iCurveLineWidth);
   pen.setColor(myColor);

// set the line style
   
   if (plotLine == PLOT_LINE_DASHED) pen.setStyle(Qt::DashLine);
   else if (plotLine == PLOT_LINE_DOTTED) pen.setStyle(Qt::DotLine);
   else if (plotLine == PLOT_LINE_DASH_DOT) pen.setStyle(Qt::DashDotLine);
   else if (plotLine == PLOT_LINE_DASH_DOT_DOT) pen.setStyle(Qt::DashDotDotLine);
   else pen.setStyle(Qt::SolidLine);
   painter.setPen(pen);

// y-axis scale type

   bool useLog = false;
   if (scaleTypeY[indexPosition(iRow, iCol)] == Log) useLog = true;

   double xAxisMin, xAxisMax, yAxisMin, yAxisMax;
   if (useLog)
   {
      xAxisMin = axisLimits[indexPosition(iRow, iCol)][0][0];
      xAxisMax = axisLimits[indexPosition(iRow, iCol)][0][1];
      yAxisMin = SafeLog(axisLimits[indexPosition(iRow, iCol)][1][0]);
      yAxisMax = SafeLog(axisLimits[indexPosition(iRow, iCol)][1][1]);
      yValue = SafeLog(yValue);
   }
   else
   {
      xAxisMin = axisLimits[indexPosition(iRow, iCol)][0][0];
      xAxisMax = axisLimits[indexPosition(iRow, iCol)][0][1];
      yAxisMin = axisLimits[indexPosition(iRow, iCol)][1][0];
      yAxisMax = axisLimits[indexPosition(iRow, iCol)][1][1];
      yValue = yValue;
   }

// plot line

   double scaleFactorY = myPlotHeight/(yAxisMax - yAxisMin);
   yValue = myPlotHeight - scaleFactorY*(yValue - yAxisMin) + myStartY;

   if (yValue > yAxisMin && yValue < yAxisMax)
      painter.drawLine(QLineF(myStartX, yValue, myStartX + myPlotWidth, yValue));

}

//-------------------------------------------------------------------------------------------------
//
// draw data onto a plot
//
//-------------------------------------------------------------------------------------------------

void QsiPlot::drawCurve(int iRow, int iCol, QColor myColor, int nPoints, double *xData, double *yData, 
                        QString name, int plotSymbol)
{

   double myPlotWidth = plotWidths[indexPosition(iRow, iCol)];
   double myPlotHeight = plotHeights[indexPosition(iRow, iCol)];
   double myStartX = plotStartsX[indexPosition(iRow, iCol)];
   double myStartY = plotStartsY[indexPosition(iRow, iCol)];

   myOwnColor = myColor;
   myOwnName = name;

   symbolToPlot->fill(QColor(255, 255, 255, 0).rgba());
   if (plotSymbol == PLOT_CIRCLE)
   {
      QPainter painter1(symbolToPlot);
      QPen pena(myColor);
      pena.setWidth(1);
      painter1.setPen(pena);
      painter1.drawEllipse(QRectF(0., 0., 6., 6.));
   } 
   else if (plotSymbol == PLOT_CIRCLE_FILLED)
   {
      QPainter painter1(symbolToPlot);
      QPen pena(myColor);
      pena.setWidth(1);
      painter1.setPen(pena);
      painter1.setBrush(QBrush(myOwnColor, Qt::SolidPattern));
      painter1.drawEllipse(QRectF(0., 0., 6., 6.));
   }
   else if (plotSymbol == PLOT_CROSS)
   {
      QPainter painter1(symbolToPlot);
      QPen pena(myColor);
      pena.setWidth(1);
      pena.setStyle(Qt::SolidLine);
      painter1.setPen(pena);
      painter1.drawLine(QLineF(0., 0., 6., 6.));
      painter1.drawLine(QLineF(0., 6., 6., 0.));
   }
   else if (plotSymbol == PLOT_PLUS)
   {
      QPainter painter1(symbolToPlot);
      QPen pena(myColor);
      pena.setWidth(1);
      pena.setStyle(Qt::SolidLine);
      painter1.setPen(pena);
      painter1.drawLine(QLineF(0, 3.5, 6., 3.5));
      painter1.drawLine(QLineF(3.5, 0., 3.5, 6.));
   }
   else if (plotSymbol == PLOT_SQUARE)
   {
      QPainter painter1(symbolToPlot);
      QPen pena(myColor);
      pena.setWidth(1);
      pena.setStyle(Qt::SolidLine);
      painter1.setPen(pena);
      painter1.drawRect(QRectF(0., 0., 6., 6.));
   }
   else if (plotSymbol == PLOT_SQUARE_FILLED)
   {
      QPainter painter1(symbolToPlot);
      QPen pena(myColor);
      pena.setWidth(1);
      pena.setStyle(Qt::SolidLine);
      painter1.setPen(pena);
      painter1.setBrush(QBrush(myOwnColor, Qt::SolidPattern));
      painter1.drawRect(QRectF(0., 0., 6., 6.));
   }

   bool useLog = false;
   if (scaleTypeY[indexPosition(iRow, iCol)] == Log) useLog = true;
   
   bool useLogX = false;
   if (scaleTypeX[indexPosition(iRow, iCol)] == Log) useLogX = true;

   QPainter painter(plot);
   QPen pen;
   pen.setWidth(iCurveLineWidth);
   pen.setColor(myColor);

// set the line style

   if (plotSymbol == PLOT_LINE_DASHED) pen.setStyle(Qt::DashLine);
   else if (plotSymbol == PLOT_LINE_DOTTED) pen.setStyle(Qt::DotLine);
   else if (plotSymbol == PLOT_LINE_DASH_DOT) pen.setStyle(Qt::DashDotLine);
   else if (plotSymbol == PLOT_LINE_DASH_DOT_DOT) pen.setStyle(Qt::DashDotDotLine);
   else pen.setStyle(Qt::SolidLine);
   painter.setPen(pen);

// set anti-aliasing if necessary

#ifdef Q_WS_MAC
//   painter.setRenderHint(QPainter::Antialiasing);
#endif

// ensure we can only draw within the panel frame

   painter.setClipRect(QRectF(myStartX, myStartY, myPlotWidth, myPlotHeight));

// convert the data to log if necessary 

   double *dx = new double[nPoints];
   double *dy = new double[nPoints];

   for (int i=0; i<nPoints; i++)
   {
      *(dx + i) = (useLogX) ? SafeLog(xData[i]) : xData[i];
      *(dy + i) = (useLog) ? SafeLog(yData[i]) : yData[i];
      *(dy + i) = (fabs(yData[i]) < MaxAllowedValue) ? *(dy + i) : FillValue;
   }

   double xAxisMin, xAxisMax, yAxisMin, yAxisMax;
   if (useLog)
   {
      yAxisMin = SafeLog(axisLimits[indexPosition(iRow, iCol)][1][0]);
      yAxisMax = SafeLog(axisLimits[indexPosition(iRow, iCol)][1][1]);
   }
   else
   {
      yAxisMin = axisLimits[indexPosition(iRow, iCol)][1][0];
      yAxisMax = axisLimits[indexPosition(iRow, iCol)][1][1];
   }
   
   if (useLogX)
   {
      xAxisMin = SafeLog(axisLimits[indexPosition(iRow, iCol)][0][0]);
      xAxisMax = SafeLog(axisLimits[indexPosition(iRow, iCol)][0][1]);   
   }
   else
   {
      xAxisMin = axisLimits[indexPosition(iRow, iCol)][0][0];
      xAxisMax = axisLimits[indexPosition(iRow, iCol)][0][1];  
   }

// plot all data

   for (int i=1; i<nPoints; i++)
   {
      double xValue = *(dx + i);
      double yValue = *(dy + i);
      double xValuePrev = *(dx + i - 1);
      double yValuePrev = *(dy + i - 1);
    
      if (yValue != FillValue && yValuePrev != FillValue)
      {
         if (xValue < xAxisMin) xValue = xAxisMin;
         if (xValue > xAxisMax) xValue = xAxisMax;
         if (yValue < yAxisMin) yValue = yAxisMin;
         if (yValue > yAxisMax) yValue = yAxisMax;

         if (xValuePrev < xAxisMin) xValuePrev = xAxisMin;
         if (xValuePrev > xAxisMax) xValuePrev = xAxisMax;
         if (yValuePrev < yAxisMin) yValuePrev = yAxisMin;
         if (yValuePrev > yAxisMax) yValuePrev = yAxisMax;
      
         double scaleFactorX = myPlotWidth/(xAxisMax - xAxisMin);
         double scaleFactorY = myPlotHeight/(yAxisMax - yAxisMin);

         xValue = scaleFactorX*(xValue - xAxisMin) + myStartX;
         yValue = myPlotHeight - scaleFactorY*(yValue - yAxisMin) + myStartY;

         xValuePrev = scaleFactorX*(xValuePrev - xAxisMin) + myStartX;
         yValuePrev = myPlotHeight - scaleFactorY*(yValuePrev - yAxisMin) + myStartY;

         if ((xValuePrev == myStartX && xValue == myStartX) || 
             (xValuePrev == myStartX+myPlotWidth && xValue == myStartX+myPlotWidth) ||
             (yValuePrev == myStartY && yValue == myStartY))
         {

         }
         else
         {
            if (plotSymbol == PLOT_LINE || plotSymbol == PLOT_LINE_DASHED ||
                plotSymbol == PLOT_LINE_DOTTED)		 
	       painter.drawLine(QLineF(xValuePrev, yValuePrev, xValue, yValue));
            else 
	       painter.drawImage(QRectF(xValue-3.5, yValue-3.5, symbolWidth, symbolHeight),
	          *symbolToPlot, QRectF(0., 0., symbolWidth, symbolHeight));
         }
      }
   }

   delete [] dx;
   delete [] dy;
}

//-------------------------------------------------------------------------------------------------
//
// draw a label on the x-axis
//
//-------------------------------------------------------------------------------------------------

void QsiPlot::setLabelX(int iRow, int iCol, QString label)
{
   double myPlotWidth = plotWidths[indexPosition(iRow, iCol)];
   double myPlotHeight = plotHeights[indexPosition(iRow, iCol)];
   double myStartX = plotStartsX[indexPosition(iRow, iCol)];
   double myStartY = plotStartsY[indexPosition(iRow, iCol)];

   QPainter painter(plot);
   painter.setPen(Qt::black);
   painter.translate(myStartX + myPlotWidth/2.0, myStartY + myPlotHeight + 30);
   painter.setRenderHint(QPainter::TextAntialiasing);
   drawRichText(label, fontLabels, ALIGN_CENTRE, &painter, 2);
   painter.resetMatrix();
}

//-------------------------------------------------------------------------------------------------
//
// draw a label on the y-axis
//
//-------------------------------------------------------------------------------------------------

void QsiPlot::setLabelY(int iRow, int iCol, QString label)
{
   double myPlotHeight = plotHeights[indexPosition(iRow, iCol)];
   double myStartY = plotStartsY[indexPosition(iRow, iCol)];

   QPainter painter(plot);
   painter.setPen(Qt::black);
   painter.translate(15, myPlotHeight/2.0 + myStartY);
   painter.rotate(270);
   painter.setRenderHint(QPainter::TextAntialiasing);
   drawRichText(label, fontLabels, ALIGN_CENTRE, &painter, 2);
   painter.resetMatrix();

}

//-------------------------------------------------------------------------------------------------
//
// set the number of x-axis ticks
//
//-------------------------------------------------------------------------------------------------

void QsiPlot::setNumTicksX(int iRow, int iCol, double durMajor, int numMinor)
{
   double myPlotWidth = plotWidths[indexPosition(iRow, iCol)];
   double myPlotHeight = plotHeights[indexPosition(iRow, iCol)];
   double myStartX = plotStartsX[indexPosition(iRow, iCol)];
   double myStartY = plotStartsY[indexPosition(iRow, iCol)];

   double posY1 = myPlotHeight + myStartY;
   double posY2 = myPlotHeight + myStartY - iPlotTickMajor;
   double posY3 = myPlotHeight + myStartY - iPlotTickMinor;

   double posY1t = myStartY;
   double posY2t = myStartY + iPlotTickMajor;
   double posY3t = myStartY + iPlotTickMinor;

   QPainter painter(plot);
   QPen pen;
   pen.setWidth(iPlotLineWidth);
   painter.setPen(pen);

   double scale = myPlotWidth/(axisLimits[indexPosition(iRow, iCol)][0][1] -
                               axisLimits[indexPosition(iRow, iCol)][0][0]);

   double durMajorPlot = scale*durMajor;
   
   bool onlyOnce = false, onlyOnceNext = false;
   double xPosDate = -1, xPosDateNext = -1, prevValue = -1;

   int numMajor = (int)myPlotWidth/durMajorPlot + 1;

   QSizeF fontPointX;

// decide whether to use scientific notation or not

   bool useExponent = false;
   if (axisLimits[indexPosition(iRow, iCol)][1][0] >= 10000. ||
       axisLimits[indexPosition(iRow, iCol)][1][1] >= 10000. ||
       (axisLimits[indexPosition(iRow, iCol)][1][0] <= 1.0e-04 && axisLimits[indexPosition(iRow, iCol)][1][0] >= 1.0e-08) ||
       (axisLimits[indexPosition(iRow, iCol)][1][1] <= 1.0e-04 && axisLimits[indexPosition(iRow, iCol)][1][1] >= 1.0e-08))
   {
      useExponent = true;
   }

   if (scaleTypeX[indexPosition(iRow, iCol)] != Log)
   {
   
//--------------------------------------------------------------------   
//----------------- for linear or time-series x-axis -----------------
//--------------------------------------------------------------------   

      for (int i=0; i<=numMajor; i++)
      {
         double posX = (double)i*durMajorPlot + myStartX;
         if (posX <= myStartX + myPlotWidth)
         {
            painter.drawLine(QLineF(posX, posY1, posX, posY2));
            painter.drawLine(QLineF(posX, posY1t, posX, posY2t));
            if (xGrid)
            {
               painter.setPen(Qt::DotLine);
               painter.drawLine(QLineF(posX, posY1, posX, posY1t));
               painter.setPen(Qt::SolidLine);
            }
            if (showValueX[indexPosition(iRow, iCol)] == pTrue)
            {
               double realValue = getValueFromPositionX(iRow, iCol, posX);
               QString text;
               if (plotType[indexPosition(iRow, iCol)] != TimeSeries && plotType[indexPosition(iRow, iCol)] != Spectrogram)
               {
                  text = QString("%1").arg(realValue);
               }   
               else
               {
                  double realValueTmp = (realValue >= 86400.0) ? realValue - 86400.0 : realValue;
                  text = toTimeString((int)(1000.0*realValueTmp+0.5), xAxisTimeString);
               }
               painter.setFont(fontTicksX);
               painter.translate(posX, myStartY + myPlotHeight + 15);
               fontPointX = drawRichText(text, fontTicksX, ALIGN_CENTRE, &painter);
               painter.resetMatrix();
               if (!onlyOnce)
               {
                  onlyOnce = true;
                  xPosDate = posX;	
               }
               if (!onlyOnceNext && realValue >= 86400.0 && posX > prevValue && prevValue != -1)
               {
                  onlyOnceNext = true;
                  xPosDateNext = posX;	
               }
               prevValue = posX;
            }

            if (i<numMajor)
            {
               for (int j=0; j<numMinor; j++)
               {
                  double posXprev = (double)(i-1)*durMajorPlot + myStartX;
                  double posXminor = (double)j*(posX - posXprev)/(double)numMinor + posX;
                  if (posXminor <= myStartX + myPlotWidth)
	          {
                     painter.drawLine(QLineF(posXminor, posY1, posXminor, posY3));
                     painter.drawLine(QLineF(posXminor, posY1t, posXminor, posY3t));
	          }  
               }
            }
         }
      }
   }
   else
   {
   
//--------------------------------------------------   
//----------------- for log x-axis ----------------- 
//-------------------------------------------------- 

// x-axis limits & scale factor

      double xAxisMin = SafeLog(axisLimits[indexPosition(iRow, iCol)][0][0]);
      double xAxisMax = SafeLog(axisLimits[indexPosition(iRow, iCol)][0][1]);
      double scaleFactorX = myPlotWidth/(xAxisMax - xAxisMin);

// plot

      int iNumberWritten = 0;
      for (int i=-8; i<8; i++)
      {
         double xValue = (double)i;
         double posX = scaleFactorX*(xValue - xAxisMin) + myStartX;

// major ticks and labels

         if (xValue >= xAxisMin && xValue <= xAxisMax)
         {
            if (xValue > xAxisMin && xValue < xAxisMax)
            {
               painter.drawLine(QLineF(posX, posY1, posX, posY2));
               painter.drawLine(QLineF(posX, posY1t, posX, posY2t));
            }
            painter.setFont(fontTicks);
            double realValue = pow(10.0, (double)i);
            QString text;
            if (useExponent) text = QString("%1").arg(realValue, 0, 'e', 0);
            else text = QString("%1").arg(realValue);
            iNumberWritten++;
            painter.translate(posX, myStartY + myPlotHeight + 15);
            drawRichText(text, fontTicks, ALIGN_CENTRE, &painter, 1);
            painter.resetMatrix();

         }
      }

// minor ticks

      for (int i=-8; i<8; i++)
      {
         double xValue = (double)i;
         double posX = scaleFactorX*(xValue - xAxisMin) + myStartX;

         int wroteOne = 0;
         for (int j=2; j<10; j++)
         {
            xValue = SafeLog((double)j*pow(10.0, (double)i));
            posX = scaleFactorX*(xValue - xAxisMin) + myStartX;
            if (xValue > xAxisMin && xValue < xAxisMax)
            {
               painter.drawLine(QLineF(posX, posY1, posX, posY3));
               painter.drawLine(QLineF(posX, posY1t, posX, posY3t));
               if (iNumberWritten == 0 || (iNumberWritten < 2 && wroteOne == 0))
               {
                  double realValue = (double)j*pow(10.0, (double)i);
                  QString text;
                  if (useExponent) text = QString("%1").arg(realValue, 0, 'e', 3);
                  else text = QString("%1").arg(realValue);
		  painter.setFont(fontTicks);
                  QRect fontBoundingRect = QFontMetrics(fontTicks).boundingRect(text);   
                  painter.translate(posX, myStartY + myPlotHeight + 15);
                  drawRichText(text, fontTicks, ALIGN_CENTRE, &painter, 1);
                  painter.resetMatrix();
                  wroteOne++;

               }
            }
         }
      }

   }
   
// write the date to the plot if necessary

   if (plotType[indexPosition(iRow, iCol)] != Scalar && showValueX[indexPosition(iRow, iCol)] == pTrue)
   {

      painter.setFont(fontTicksX);
      painter.translate(xPosDate, myStartY + myPlotHeight + 11 + fontPointX.height());
      drawRichText(initialDate.toString("yyyy-MM-dd"), fontTicksX, ALIGN_CENTRE, &painter);
      painter.resetMatrix();   
      if (onlyOnceNext)
      {
         painter.setFont(fontTicksX);
         painter.translate(xPosDateNext, myStartY + myPlotHeight + 11 + fontPointX.height());
         drawRichText(initialDate.addDays(1).toString("yyyy-MM-dd"), fontTicksX, ALIGN_CENTRE, &painter);
         painter.resetMatrix();         	
      }                      
   }  
}

//-------------------------------------------------------------------------------------------------
//
// set the number of y-axis ticks
//
//-------------------------------------------------------------------------------------------------

void QsiPlot::setNumTicksY(int iRow, int iCol, int numMajor, int numMinor)
{
   double myPlotWidth = plotWidths[indexPosition(iRow, iCol)];
   double myPlotHeight = plotHeights[indexPosition(iRow, iCol)];
   double myStartX = plotStartsX[indexPosition(iRow, iCol)];
   double myStartY = plotStartsY[indexPosition(iRow, iCol)];

   double posX1 = myStartX;
   double posX2 = myStartX + iPlotTickMajor;
   double posX3 = myStartX + iPlotTickMinor;

   double posX1t = myStartX + myPlotWidth;
   double posX2t = myStartX + myPlotWidth - iPlotTickMajor;
   double posX3t = myStartX + myPlotWidth - iPlotTickMinor;
   
   double ySign = -1.0;
   
   bool dontShowNumbers = false;
   if (numCols > 1 && indexPositionReturn(iRow, iCol, QS_COL) != 0) dontShowNumbers = true;
   
#ifdef Q_WS_MAC
   ySign = -1.0;
#endif

// decide whether to use scientific notation or not

   bool useExponent = false;
   if (axisLimits[indexPosition(iRow, iCol)][1][0] >= 10000. ||
       axisLimits[indexPosition(iRow, iCol)][1][1] >= 10000. ||
       (axisLimits[indexPosition(iRow, iCol)][1][0] <= 1.0e-04 && axisLimits[indexPosition(iRow, iCol)][1][0] >= 1.0e-08) ||
       (axisLimits[indexPosition(iRow, iCol)][1][1] <= 1.0e-04 && axisLimits[indexPosition(iRow, iCol)][1][1] >= 1.0e-08))
   {
      useExponent = true;
   }
 
// paint 
   
   QPainter painter(plot);
   QPen pen;
   pen.setWidth(iPlotLineWidth);
   painter.setPen(pen);

   if (scaleTypeY[indexPosition(iRow, iCol)] != Log)
   {
      for (int i=0; i<=numMajor; i++)
      {
         double posY = (double)i*myPlotHeight/(double)numMajor + myStartY;
         painter.drawLine(QLineF(posX1, posY, posX2, posY));
         painter.drawLine(QLineF(posX1t, posY, posX2t, posY));

         if (yGrid)
         {
            painter.setPen(Qt::DotLine);
            painter.drawLine(QLineF(posX1, posY, posX1t, posY));
            painter.setPen(Qt::SolidLine);
         }

         painter.setFont(fontTicks);
         double realValue = getValueFromPositionY(iRow, iCol, posY);
         if (fabs(realValue) < 1.0e-10) realValue = 0.0;
         QString text;
         if (useExponent) text = QString("%1").arg(realValue, 0, 'e', 0);
         else text = QString("%1").arg(realValue);
         painter.translate(myStartX - gapBetweenAxisAndTick, posY);
         if (!dontShowNumbers) drawRichText(text, fontTicks, ALIGN_RIGHT, &painter);
         painter.resetMatrix();

         if (i < numMajor)
         {
            for (int j=0; j<numMinor; j++)
            {
               double posYprev = (double)(i-1)*myPlotHeight/(double)numMajor + myStartY;
               double posYminor = (double)j*(posY - posYprev)/(double)numMinor + posY;
               painter.drawLine(QLineF(posX1, posYminor, posX3, posYminor));
               painter.drawLine(QLineF(posX1t, posYminor, posX3t, posYminor));
            }
         }
      }
   }
   else
   {

// y-axis limits & scale factor

      double yAxisMin = SafeLog(axisLimits[indexPosition(iRow, iCol)][1][0]);
      double yAxisMax = SafeLog(axisLimits[indexPosition(iRow, iCol)][1][1]);
      double scaleFactorY = myPlotHeight/(yAxisMax - yAxisMin);

// plot

      int iNumberWritten = 0;
      for (int i=-8; i<8; i++)
      {
         double yValue = (double)i;
         double posY = myPlotHeight - scaleFactorY*(yValue - yAxisMin) + myStartY;

// major ticks and labels

         if (yValue >= yAxisMin && yValue <= yAxisMax)
         {
            if (yValue > yAxisMin && yValue < yAxisMax)
            {
               painter.drawLine(QLineF(posX1, posY, posX2, posY));
	       painter.drawLine(QLineF(posX1t, posY, posX2t, posY));
               if (yGrid)
               {
                  painter.setPen(Qt::DotLine);
                  painter.drawLine(QLineF(posX1, posY, posX1t, posY));
                  painter.setPen(Qt::SolidLine);
               }
            }
            painter.setFont(fontTicks);
            double realValue = pow(10.0, (double)i);
            if (fabs(realValue) < 1.0e-10) realValue = 0.0;
            QString text;
            if (useExponent) text = QString("%1").arg(realValue, 0, 'e', 0);
            else text = QString("%1").arg(realValue);
            iNumberWritten++;
            painter.translate(myStartX - gapBetweenAxisAndTick, posY);
            if (!dontShowNumbers) drawRichText(text, fontTicks, ALIGN_RIGHT, &painter, 1);
            painter.resetMatrix();

         }
      }

// minor ticks

      for (int i=-8; i<8; i++)
      {
         double yValue = (double)i;
         double posY = myPlotHeight - scaleFactorY*(yValue - yAxisMin) + myStartY;

         int wroteOne = 0;
         for (int j=2; j<10; j++)
         {
            yValue = SafeLog((double)j*pow(10.0, (double)i));
            posY = myPlotHeight - scaleFactorY*(yValue - yAxisMin) + myStartY;
            if (yValue > yAxisMin && yValue < yAxisMax)
            {
               painter.drawLine(QLineF(posX1, posY, posX3, posY));
               painter.drawLine(QLineF(posX1t, posY, posX3t, posY));
               if (iNumberWritten == 0 || (iNumberWritten < 2 && wroteOne == 0))
               {
                  double realValue = (double)j*pow(10.0, (double)i);
                  if (fabs(realValue) < 1.0e-10) realValue = 0.0;
                  QString text;
                  if (useExponent) text = QString("%1").arg(realValue, 0, 'e', 3);
                  else text = QString("%1").arg(realValue);
		  painter.setFont(fontTicks);
                  QRect fontBoundingRect = QFontMetrics(fontTicks).boundingRect(text);   
                  painter.translate(myStartX - gapBetweenAxisAndTick, posY);
                  if (!dontShowNumbers) drawRichText(text, fontTicks, ALIGN_RIGHT, &painter, 1);
                  painter.resetMatrix();
                  wroteOne++;

               }
            }
         }
      }
   }
}

//-------------------------------------------------------------------------------------------------
//
// specify whether x-axis values should be shown
//
//-------------------------------------------------------------------------------------------------

void QsiPlot::showValuesX(int iRow, int iCol, int value)
{
   showValueX[indexPosition(iRow, iCol)] = value;
}

//-------------------------------------------------------------------------------------------------
//
// convert a y-position on the plot to a real data value
//
//-------------------------------------------------------------------------------------------------

double QsiPlot::getValueFromPositionY(int iRow, int iCol, double in)
{
   double myPlotHeight = plotHeights[indexPosition(iRow, iCol)];
   double myStartY = plotStartsY[indexPosition(iRow, iCol)];

   double scale = myPlotHeight/(axisLimits[indexPosition(iRow, iCol)][1][1] -
                                axisLimits[indexPosition(iRow, iCol)][1][0]);

   double out = axisLimits[indexPosition(iRow, iCol)][1][0] + (myPlotHeight + myStartY - in)/scale;
   return(out);

}

//-------------------------------------------------------------------------------------------------
//
// convert an x-position on the plot to a real data value
//
//-------------------------------------------------------------------------------------------------

double QsiPlot::getValueFromPositionX(int iRow, int iCol, double in)
{
   double myPlotWidth = plotWidths[indexPosition(iRow, iCol)];
   double myStartX = plotStartsX[indexPosition(iRow, iCol)];

   double scale = myPlotWidth/(axisLimits[indexPosition(iRow, iCol)][0][1] -
                               axisLimits[indexPosition(iRow, iCol)][0][0]);

   double out = axisLimits[indexPosition(iRow, iCol)][0][0] + (in - myStartX)/scale;
   return(out);

}

//-------------------------------------------------------------------------------------------------
//
// set the time format used for x-axis time strings
//
//-------------------------------------------------------------------------------------------------

void QsiPlot::setTimeStringFormat(QString format)
{
   timeStringFormat = format;
   if (format == "hh:mm") xAxisTimeString = 1;
   else xAxisTimeString = 0;
}

//-------------------------------------------------------------------------------------------------
//
// draw label on plot
//
//-------------------------------------------------------------------------------------------------

void QsiPlot::drawLabel(QString text)
{
   if (!noShowLabel)
   {
   QPainter painter(plot);
   painter.translate(QPointF(imageWidthPlot*0.99, imageHeightPlot*0.98));
   painter.setPen(QColor("gainsboro"));
   drawRichText(text, fontPL, ALIGN_RIGHT, &painter);
   painter.resetMatrix();
   }
}

//-------------------------------------------------------------------------------------------------
//
// save the current plot to a file
//
//-------------------------------------------------------------------------------------------------

void QsiPlot::saveToFile(QString fileName, QString text)
{
   if (text.size() > 0) plot->setText("Description", text);
   plot->save(fileName, 0, 10);

//      QPrinter printer(QPrinter::HighResolution);
//      printer.setOutputFileName(fileName);
//      printer.setOrientation(QPrinter::Landscape);
//      QPainter painter(&printer);
//      painter.drawImage(QPoint(0,0), *plot);
//      painter.end();
}

//-------------------------------------------------------------------------------------------------
//
// set the font to be used for all labels
//
//-------------------------------------------------------------------------------------------------

void QsiPlot::setGlobalFontLabels(QFont font)
{
   fontLabels = font;
}

//-------------------------------------------------------------------------------------------------
//
// set the font to be used for all ticks
//
//-------------------------------------------------------------------------------------------------

void QsiPlot::setGlobalFontTicks(QFont font)
{
   fontTicks = font;
}

//-------------------------------------------------------------------------------------------------
//
// set the font to be used for the title
//
//-------------------------------------------------------------------------------------------------

void QsiPlot::setGlobalFontTitle(QFont font)
{
   fontTitle = font;
}

//-------------------------------------------------------------------------------------------------
//
// set the font to be used for legends
//
//-------------------------------------------------------------------------------------------------

void QsiPlot::setGlobalFontLegends(QFont font)
{
   fontLegends = font;
}

//-------------------------------------------------------------------------------------------------
//
// set the font to be used for the colour box 
//
//-------------------------------------------------------------------------------------------------

void QsiPlot::setGlobalFontColourBox(QFont font)
{
   fontColourBox = font;
}

//-------------------------------------------------------------------------------------------------
//
// determine number of ticks and min/max values for y-axis
//
//-------------------------------------------------------------------------------------------------

void QsiPlot::getAutoScaleY(double min, double max, int *numMajorTicks, int *numMinorTicks,
                            double *valMin, double *valMax, int scaleType)
{
   
   double tickSize;
   determineYaxisTicks(min, max, valMin, valMax, &tickSize, scaleType);
   double numRaw = (*valMax - *valMin)/tickSize;

   *numMajorTicks = (int)(numRaw+0.5);

   if ((int)tickSize % 10 == 0 && numRaw < 10)
   {
      *numMinorTicks = 5;
   }
   else if ((int)tickSize % 2 == 0)
   {
      *numMinorTicks = 2;
   }
   else if ((int)tickSize % 6 == 0)
   {
      *numMinorTicks = 6;
   }
   else if ((int)tickSize % 5 == 0)
   {
      *numMinorTicks = 5;
   }
   else if ((int)tickSize % 4 == 0)
   {
      *numMinorTicks = 4;
   }
   else if ((int)tickSize % 3 == 0)
   {
      *numMinorTicks = 3;
   }
   else if ((int)tickSize % 8 == 0)
   {
      *numMinorTicks = 8;
   }
   else
   {
      *numMinorTicks = 1;
   }   
}

//-------------------------------------------------------------------------------------------------
//
// set the location of the legend
//
//-------------------------------------------------------------------------------------------------

void QsiPlot::setLegendLocation(int location)
{
   legendLocation = location;
}

//-------------------------------------------------------------------------------------------------
//
// show the legend for a particular plot
//
//-------------------------------------------------------------------------------------------------

void QsiPlot::drawPlotTitle(int iRow, int iCol, QString myPlotPanelTitle, int pos)
{
   double myPlotWidth = plotWidths[indexPosition(iRow, iCol)];
   double myStartX = plotStartsX[indexPosition(iRow, iCol)];
   double myStartY = plotStartsY[indexPosition(iRow, iCol)];
   
   double startLocationX = (pos == 0) ? myStartX + 8 : myStartX + myPlotWidth - 8;
   double startLocationY = myStartY + 15;

   QPainter painter(plot);
   painter.translate(QPointF(startLocationX, startLocationY));

   if (pos == 0)
   {
      drawRichText(myPlotPanelTitle, fontPanelTitle, ALIGN_LEFT, &painter, 2);
   }
   else
   {
      drawRichText(myPlotPanelTitle, fontSubTitle, ALIGN_RIGHT, &painter, 2);
   }
}

//-------------------------------------------------------------------------------------------------
//
// show the legend for a particular plot
//
//-------------------------------------------------------------------------------------------------

void QsiPlot::showLegend(int iRow, int iCol, int num, int plotSymbol)
{
   double myPlotWidth = plotWidths[indexPosition(iRow, iCol)];
   double myStartX = plotStartsX[indexPosition(iRow, iCol)];
   double myStartY = plotStartsY[indexPosition(iRow, iCol)];
   
   double startLocationX = myPlotWidth + myStartX + 8;
   double startLocationY = myStartY + 15;
     
   if (legendLocation == Outside)
   {
      startLocationX = myPlotWidth + myStartX + 8;
   } 
   else if (legendLocation == TopLeft)
   {
      startLocationX = myStartX + 15; 
   }

   double lineLength = 20;

   QPen pen;
   pen.setWidth(iPlotLineWidth);
   pen.setColor(myOwnColor);

   QPainter painter(plot);
   painter.setFont(fontLegends);

   QRect fontBoundingRect = QFontMetrics(fontLegends).boundingRect(myOwnName);
   double startLocationXt = startLocationX + lineLength + 8;

   startLocationY = startLocationY + (double)num*((double)fontBoundingRect.height())*1.0;
   double startLocationYY = startLocationY - ((double)fontBoundingRect.height())/4.0;

   painter.translate(QPointF(startLocationXt, startLocationY));
   drawRichText(myOwnName, fontLegends, ALIGN_LEFT, &painter);
   painter.resetMatrix();

   painter.setPen(pen);

// set the line style

   if (plotSymbol == PLOT_LINE_DASHED) pen.setStyle(Qt::DashLine);
   else if (plotSymbol == PLOT_LINE_DOTTED) pen.setStyle(Qt::DotLine);
   else if (plotSymbol == PLOT_LINE_DASH_DOT) pen.setStyle(Qt::DashDotLine);
   else if (plotSymbol == PLOT_LINE_DASH_DOT_DOT) pen.setStyle(Qt::DashDotDotLine);
   else pen.setStyle(Qt::SolidLine);
   painter.setPen(pen);

   symbolToPlot->fill(QColor(255, 255, 255, 0).rgba());
   if (plotSymbol == PLOT_CIRCLE)
   {
      QPainter painter1(symbolToPlot);
      QPen pena(myOwnColor);
      pena.setWidth(1);
      painter1.setPen(pena);
      painter1.drawEllipse(QRectF(0., 0., 6., 6.));
   } 
   else if (plotSymbol == PLOT_CIRCLE_FILLED)
   {  
      QPainter painter1(symbolToPlot);
      QPen pena(myOwnColor);
      pena.setWidth(1);
      painter1.setPen(pena);
      painter1.setBrush(QBrush(myOwnColor, Qt::SolidPattern));
      painter1.drawEllipse(QRectF(0., 0., 6., 6.));
   }
   else if (plotSymbol == PLOT_CROSS)
   {  
      QPainter painter1(symbolToPlot);
      QPen pena(myOwnColor);
      pena.setWidth(1);
      pena.setStyle(Qt::SolidLine);
      painter1.setPen(pena);
      painter1.drawLine(QLine(0, 0, 6, 6));
      painter1.drawLine(QLine(0, 6, 6, 0));
   }
   else if (plotSymbol == PLOT_PLUS)
   {  
      QPainter painter1(symbolToPlot);
      QPen pena(myOwnColor);
      pena.setWidth(1);
      pena.setStyle(Qt::SolidLine);
      painter1.setPen(pena);
      painter1.drawLine(QLineF(0, 3.5, 6., 3.5));
      painter1.drawLine(QLineF(3.5, 0., 3.5, 6.));
   }
   else if (plotSymbol == PLOT_SQUARE)
   {  
      QPainter painter1(symbolToPlot);
      QPen pena(myOwnColor);
      pena.setWidth(1);
      pena.setStyle(Qt::SolidLine);
      painter1.setPen(pena);
      painter1.drawRect(QRectF(0., 0., 6., 6.));
   }
   else if (plotSymbol == PLOT_SQUARE_FILLED)
   {  
      QPainter painter1(symbolToPlot);
      QPen pena(myOwnColor);
      pena.setWidth(1);
      pena.setStyle(Qt::SolidLine);
      painter1.setPen(pena);
      painter1.setBrush(QBrush(myOwnColor, Qt::SolidPattern));
      painter1.drawRect(QRectF(0., 0., 6., 6.));
   }

   if (plotSymbol == PLOT_LINE || plotSymbol == PLOT_LINE_DASHED || plotSymbol == PLOT_LINE_DOTTED ||
       plotSymbol == PLOT_LINE_DASH_DOT || plotSymbol == PLOT_LINE_DASH_DOT_DOT)
   {
      painter.drawLine(QLineF(startLocationX, startLocationYY, startLocationX + lineLength, startLocationYY));
   }
   else if (plotSymbol == PLOT_CIRCLE || plotSymbol == PLOT_CIRCLE_FILLED ||
            plotSymbol == PLOT_CROSS || plotSymbol == PLOT_PLUS ||
            plotSymbol == PLOT_SQUARE || plotSymbol == PLOT_SQUARE_FILLED)
   {
      painter.drawImage(QRectF(startLocationX+lineLength/2.0-3.5, startLocationYY-3.5, symbolWidth, symbolHeight),
                        *symbolToPlot, QRectF(0., 0., symbolWidth, symbolHeight));

   }
}

//----------------------------------------------------------------------------------------
// 
// safe log 10
//
//----------------------------------------------------------------------------------------

double QsiPlot::SafeLog(double x)
{  
   return (x < 1.0e-33) ? -33.0 : log10(x);
}

//----------------------------------------------------------------------------------------
// 
// determine y-axis lower & upper limits, major tick width
//
//----------------------------------------------------------------------------------------

void QsiPlot::determineYaxisTicks(double yMin, double yMax,
                                  double *yAxisMin, double *yAxisMax, double *tics,
                                  int scaleType)
{

// determine the tick size

   double totalHeight = yMax - yMin; // total height of y-axis
   double posns = totalHeight/6.0;

   bool done = false;
   int order = -20;
   while (order < 30 && !done)
   {
      if (posns <= 1.0*pow(10.0, (double)order))
      {
         *tics = 1.0*pow(10.0, (double)order);    /* eg 0,1,2,... */
         done = true;
      }
      else if (posns <= 2.0*pow(10.0, (double)order))
      {
         *tics = 2.0*pow(10.0, (double)order);    /* eg 0,2,4,6,... */
         done = true;
      }
      else if (posns <= 5.0*pow(10.0, (double)order))
      {
         *tics = 5.0*pow(10.0, (double)order);    /* eg 0,5,10,15,... */
         done = true;
      }
      order++;
   }

// determine the lower & upper starting points

   if (scaleType == Linear)
   {
      if (yMin >= 0)
      {
         *yAxisMin = (int)(yMin/ *tics)* *tics;
      }
      else
      {
         *yAxisMin = -ceil(-yMin/ *tics)* *tics;
      }

      *yAxisMax = ceil(yMax/ *tics)* *tics;

   }
   else
   {
      *yAxisMin = yMin;
      *yAxisMax = yMax;
   }   
}

//-------------------------------------------------------------------------------------------------
//
// determine number of ticks and min/max values for x-axis
//
//-------------------------------------------------------------------------------------------------

void QsiPlot::getAutoScaleX(double *data, int size, double userStartTime, double userStopTime,
                            double *durationMajor, int *numMinorTicks,
                            double *valMin, double *valMax)
{
   double min = 0, max = 0;

// use either the user specified begin/end times or begin/end values from array

   if (userStartTime >= 0 && userStopTime >= 0)
   {
      min = userStartTime;
      max = userStopTime;
   }
   else if (size > 0)
   {
      min = *(data+0);
      max = *(data+size-1);
   }

// "quantize" the time values if from the array or requested by user

   if ((size == -1 || size > 0) && (userStopTime - userStartTime) > 60.0)
   {
      int bHr, bMin, bSec, bMilli;
      int eHr, eMin, eSec, eMilli;

      toTime(1000.0*min, &bHr, &bMin, &bSec, &bMilli);
      toTime(1000.0*max, &eHr, &eMin, &eSec, &eMilli);

      min = 60.0*(bMin + 60.0*bHr);
      max = 60.0*(eMin + 60.0*eHr);
   }
   
   int numMinorTicksRaw;
   determineXaxisTicks(min, max, durationMajor, &numMinorTicksRaw);	
   
   *numMinorTicks = numMinorTicksRaw;
   
   *valMin = min;
   *valMax = max;
}
                            
//----------------------------------------------------------------------------------------
// 
// determine x-axis ticks
//
//----------------------------------------------------------------------------------------

void QsiPlot::determineXaxisTicks(double stTime, double spTime, double *durationMajor, int *ticksMinor)
{
   double duration = (spTime - stTime)/60.0;   // duration in minutes
   if (duration < 10.0/60.0)
   {
      *durationMajor = 1;
      *ticksMinor = 2; 
   }
   else if ((duration >= 10.0/60.0) && (duration < 0.5))
   {
      *durationMajor = 5;
      *ticksMinor = 5; 
   }
   else if ((duration >= 0.5) && (duration < 1))
   {
      *durationMajor = 10;
      *ticksMinor = 2;
   }
   else if ((duration >= 1.0) && (duration < 2))
   {
      *durationMajor = 30;
      *ticksMinor = 6;
   }
   else if ((duration >= 2) && (duration < 10))
   {
      *durationMajor = 60;
      *ticksMinor = 2;
   }
   else if ((duration >= 10) && (duration < 15))
   {
      *durationMajor = 2*60;
      *ticksMinor = 2;
   }
   else if ((duration >= 15) && (duration < 30))
   {
      *durationMajor = 5*60;
      *ticksMinor = 10;
   }
   else if ((duration >= 30) && (duration < 60))
   {
      *durationMajor = 10*60;
      *ticksMinor = 10;
   }
   else if ((duration >= 60) && (duration < 5*60))
   {
      *durationMajor = 30*60;
      *ticksMinor = 6;
   }
   else if ((duration >= 5*60) && (duration < 11*60))
   {
      *durationMajor = 60*60;
      *ticksMinor = 6;
   }
   else
   {
      *durationMajor = 4*60*60;
      *ticksMinor = 8;
   }
}

//-------------------------------------------------------------------------------------------------
//
// draw a spectrogram
//
//-------------------------------------------------------------------------------------------------

void QsiPlot::drawSpectrogramSlice(int iRow, int iCol, double z,
                                   double xBegin, double xEnd, 
                                   double yBegin, double yEnd,
                                   double colourBarMin, double colourBarMax)
{
   double myPlotWidth = plotWidths[indexPosition(iRow, iCol)];
   double myPlotHeight = plotHeights[indexPosition(iRow, iCol)];
   double myStartX = plotStartsX[indexPosition(iRow, iCol)];
   double myStartY = plotStartsY[indexPosition(iRow, iCol)];
   
   double zval = (scaleTypeZ[indexPosition(iRow, iCol)] == Log) ? SafeLog(z) : z;

   if (scaleTypeZ[indexPosition(iRow, iCol)] == Log) 
   {
      colourBarMin = SafeLog(colourBarMin);
      colourBarMax = SafeLog(colourBarMax);
   }

   bool useLog = false;
   if (scaleTypeY[indexPosition(iRow, iCol)] == Log) useLog = true;
   
   double yAxisMin = axisLimits[indexPosition(iRow, iCol)][1][0];
   double yAxisMax = axisLimits[indexPosition(iRow, iCol)][1][1];

   if (useLog)
   {
      yAxisMin = SafeLog(yAxisMin);
      yAxisMax = SafeLog(yAxisMax);
   }
 
   double xAxisMin = axisLimits[indexPosition(iRow, iCol)][0][0];
   double xAxisMax = axisLimits[indexPosition(iRow, iCol)][0][1];
   
   if (useLog)
   {
      yBegin = SafeLog(yBegin);
      yEnd = SafeLog(yEnd);
   }

   double scaleFactorX = myPlotWidth/(xAxisMax - xAxisMin);
   double scaleFactorY = myPlotHeight/(yAxisMax - yAxisMin);
   
   if (xBegin < xAxisMin) xBegin = xAxisMin;
   if (xBegin > xAxisMax) xBegin = xAxisMax;
   if (xEnd < xAxisMin) xEnd = xAxisMin;
   if (xEnd > xAxisMax) xEnd = xAxisMax;
   
   if (yBegin < yAxisMin) yBegin = yAxisMin;
   if (yBegin > yAxisMax) yBegin = yAxisMax;
   if (yEnd < yAxisMin) yEnd = yAxisMin;
   if (yEnd > yAxisMax) yEnd = yAxisMax;
   
   double x1 = (xBegin - xAxisMin)*scaleFactorX + myStartX;
   double x2 = (xEnd - xAxisMin)*scaleFactorX + myStartX;
   double y1 = myPlotHeight - (yBegin - yAxisMin)*scaleFactorY + myStartY;
   double y2 = myPlotHeight - (yEnd - yAxisMin)*scaleFactorY + myStartY; 

   QPainter painter(plot);   

   zval = (zval < colourBarMin) ? colourBarMin : zval;
   zval = (zval > colourBarMax) ? colourBarMax : zval;
   
   double col = ((zval - (double)colourBarMin)/((double)(colourBarMax - colourBarMin)));
   col = 1.0 - col;

   if (col != 1.0)
   {
      QColor myColor = colourBarDefinition(col);
      painter.setPen(myColor);
      QBrush myBrush(myColor);
      myBrush.setStyle(Qt::SolidPattern);
      painter.setBrush(myBrush);
      painter.drawRect(QRectF(x1, y1, x2-x1, y2-y1));
   }
}

//-------------------------------------------------------------------------------------------------
// 
// draw spectrogram colour box
//    
//-------------------------------------------------------------------------------------------------
      
void QsiPlot::drawColourBox(int iRow, int iCol, double colourBarMin, double colourBarMax,
                            QString label, int iRowB)
{

   if (numCols > 1 && iRowB > 0)  // correction for multiple column plots; sorry, this is not  
   {                              // exactly general code, but it's a quick & simple for for
      iCol = numCols - 1;         // plotcf
      iRow = 0;
      iRowB = numRows - 1;
   }  

   double myPlotWidth = plotWidths[indexPosition(iRow, iCol)];
   double myPlotHeight = plotHeights[indexPosition(iRow, iCol)];
   double myStartX = plotStartsX[indexPosition(iRow, iCol)];
   double myStartY = plotStartsY[indexPosition(iRow, iCol)];

   if (iRowB != -1)
   {       
      double myPlotHeightB = plotHeights[indexPosition(iRowB, iCol)];
      double myStartYB = plotStartsY[indexPosition(iRowB, iCol)];
      myPlotHeight = myPlotHeightB + myStartYB - myStartY;
   }

   double gap = 20.;
   QPainter painter(plot);

// write colour bar label

   painter.setPen(Qt::black);
   painter.translate(QPointF(myStartX + myPlotWidth + gap/2., myPlotHeight/2. + myStartY));
   painter.rotate(270);
   drawRichText(label, fontColourBox, ALIGN_CENTRE, &painter, 2);
   painter.resetMatrix();

// draw colour bar colours

   int nmax = (int)myPlotHeight; // handle different plot sizes
   if (nmax < 200) nmax = 200;
   nmax *= 2;

   for (int i=0; i<nmax; i++)
   {
      double yLoc = (double)i*(myPlotHeight/(double)nmax);
      double yPos = myStartY + myPlotHeight - yLoc;
      QColor myColor;
      double hue = (double)(myPlotHeight - yLoc)/((double)myPlotHeight);
      painter.setPen(colourBarDefinition(hue));
      painter.drawLine(QLineF(myStartX + myPlotWidth + gap, yPos, 
                              myStartX + myPlotWidth + gap + colourBarWidth, yPos));
   }

// draw rectangle

   painter.setPen(Qt::black);
   painter.drawRect(QRectF(myStartX + myPlotWidth + gap, myStartY, colourBarWidth, myPlotHeight));

// y-axis limits & scale factor

   double yAxisMin = SafeLog(colourBarMin);
   double yAxisMax = SafeLog(colourBarMax);
   double scaleFactorY = myPlotHeight/(yAxisMax - yAxisMin);

// decide whether to use scientific notation or not

   double posX1 = myStartX + myPlotWidth + gap + colourBarWidth;
   double posX2 = myStartX + myPlotWidth + gap + colourBarWidth - iPlotTickMajor;
   double posX3 = myStartX + myPlotWidth + gap + colourBarWidth - iPlotTickMinor;

// plot

      int iNumberWritten = 0;
      for (int i=-20; i<20; i++)
      {
         double yValue = (double)i;
         double posY = myPlotHeight - scaleFactorY*(yValue - yAxisMin) + myStartY;

// major ticks and labels

         if (yValue >= yAxisMin && yValue <= yAxisMax)
         {
            if (yValue > yAxisMin && yValue < yAxisMax) painter.drawLine(QLineF(posX1, posY, posX2, posY));
            double realValue = pow(10.0, (double)i);
            QString text = QString("%1").arg(realValue, 0, 'e', 0);
            iNumberWritten++;
            painter.translate(myStartX + myPlotWidth + gap + colourBarWidth + gapBetweenAxisAndTick, posY);
            drawRichText(text, fontTicksColour, ALIGN_LEFT, &painter, 1);
            painter.resetMatrix();

         }
      }

// minor ticks

      for (int i=-20; i<20; i++)
      {
         double yValue = (double)i;
         double posY = myPlotHeight - scaleFactorY*(yValue - yAxisMin) + myStartY;

         int wroteOne = 0;
         for (int j=2; j<10; j++)
         {
            yValue = SafeLog((double)j*pow(10.0, (double)i));
            posY = myPlotHeight - scaleFactorY*(yValue - yAxisMin) + myStartY;
            if (yValue > yAxisMin && yValue < yAxisMax)
            {
               painter.drawLine(QLineF(posX1, posY, posX3, posY));
               if (iNumberWritten == 0 || (iNumberWritten < 2 && wroteOne == 0))
               {
                  double realValue = (double)j*pow(10.0, (double)i);
                  QString text = QString("%1").arg(realValue, 0, 'e', 0);
                  painter.setFont(fontTicks);
                  painter.translate(myStartX + myPlotWidth + gap + colourBarWidth + gapBetweenAxisAndTick, posY);
                  drawRichText(text, fontTicksColour, ALIGN_LEFT, &painter, 1);
                  painter.resetMatrix();
                  wroteOne++;
               }
            }
         }
      }
}

//-------------------------------------------------------------------------------------------------
// 
// draw rich text
//    
//-------------------------------------------------------------------------------------------------
      
QSizeF QsiPlot::drawRichText(QString textIn, QFont font, int alignment, QPainter *painter, int process)
{
   QSizeF returnF;

   if (useGUI)
   {

   QString text = textIn;
   if (process == 1) 
   {
      QRegExp expNorm1("^1e\\+0\\d$");
      QRegExp expNorm2("^1e\\-0\\d$");
      QRegExp expNorm1a("^1e\\+\\d\\d$");
      QRegExp expNorm2a("^1e\\-\\d\\d$");

      if (expNorm1.indexIn(textIn) == 0)
      {
         QString exponent = textIn.mid(4, 1);
         text = QString("10<sup>%1</sup>").arg(exponent);
      }  
      else if (expNorm2.indexIn(textIn) == 0)
      {
         QString exponent = textIn.mid(4, 1);
         text = QString("10<sup>-%1</sup>").arg(exponent);
      }     
      else if (expNorm1a.indexIn(textIn) == 0)
      {
         QString exponent = textIn.mid(3, 2);
         text = QString("10<sup>%1</sup>").arg(exponent);
      }  
      else if (expNorm2a.indexIn(textIn) == 0)
      {  
         QString exponent = textIn.mid(3, 2);
         text = QString("10<sup>-%1</sup>").arg(exponent);
      }

   }
   else if (process == 2)
   {
      if (textIn.contains("^"))
      {
         QString textTmp = textIn;
         bool done = false;
         while (!done)
         {
            if (!textTmp.contains("^")) done = true;
            if (!done)
            {
               int index = textTmp.indexOf("^", 0);
               QString bit = textTmp.mid(index+1, 1);
               if (bit == "{")
               {
                  int beginBr = textTmp.indexOf("{", index);
                  int endBr = textTmp.indexOf("}", beginBr);
                  if (beginBr > -1 && endBr > -1)
                  {
                     bit = textTmp.mid(beginBr+1, (endBr-beginBr));
                     text = textTmp.replace(index, (endBr-beginBr+2), "<sup>"+bit+"</sup>");
	          }
               }
               else if (bit == "-")
               {
                  bit = textTmp.mid(index+1, 2);
                  text = textTmp.replace(index, 3, "<sup>"+bit+"</sup>");
               }
               else
               text = textTmp.replace(index, 2, "<sup>"+bit+"</sup>");
            }
         }
         textTmp = text;
      }  
   }

   QTextDocument doc;
   doc.setHtml(text);
   doc.setDefaultFont(font);

   double offsetV = doc.size().height()*0.5;
   double offsetH = doc.size().width();
   if (alignment == ALIGN_LEFT) offsetH = 0.0;
   else if (alignment == ALIGN_RIGHT) offsetH = -offsetH;
   else if (alignment == ALIGN_CENTRE) offsetH = -offsetH/2.;

   painter->translate(QPointF(offsetH, -offsetV));
   doc.drawContents(painter);
   painter->translate(QPointF(-offsetH, offsetV));

   returnF = QSizeF(doc.size().width(), doc.size().height());
   }
   else
   {
      QFontMetrics fm(font);
      double textHeight = fm.boundingRect(textIn).height();
      double textWidth = fm.boundingRect(textIn).width();

      double offsetV = textHeight*0.3;
      double offsetH = textWidth;

      if (alignment == ALIGN_LEFT) offsetH = 0.0;
      else if (alignment == ALIGN_RIGHT) offsetH = -offsetH;
      else if (alignment == ALIGN_CENTRE) offsetH = -offsetH/2.;

      painter->drawText(QPointF(offsetH, offsetV), textIn);

      returnF = QSizeF(textWidth*1.1, textHeight*1.1);

   }

   return(returnF);
}

//-------------------------------------------------------------------------------------------------
//
// set energy table for Sauvaud plots
//
//-------------------------------------------------------------------------------------------------

void QsiPlot::setSauvaudEnergyTable(int iRow, int iCol, int n, float *lower, float *upper, float *centre)
{
   numEnergyBinSauvaud = n;

   double myPlotWidth = plotWidths[indexPosition(iRow, iCol)];
   double myPlotHeight = plotHeights[indexPosition(iRow, iCol)];
   double myStartX = plotStartsX[indexPosition(iRow, iCol)];
   double myStartY = plotStartsY[indexPosition(iRow, iCol)];

// setup axis limits

   bool useLog = false;
   if (scaleTypeY[indexPosition(iRow, iCol)] == Log) useLog = true;

   double yAxisMin, yAxisMax;
   if (useLog)
   {
      yAxisMin = SafeLog(axisLimits[indexPosition(iRow, iCol)][1][0]);
      yAxisMax = SafeLog(axisLimits[indexPosition(iRow, iCol)][1][1]);
   }
   else
   {
      yAxisMin = axisLimits[indexPosition(iRow, iCol)][1][0];
      yAxisMax = axisLimits[indexPosition(iRow, iCol)][1][1];
   }

// setup the painter

   QPainter painter(plot);
   painter.setRenderHint(QPainter::TextAntialiasing);

// draw lines indicating lower & upper energy bin limits; draw y-axis values
// for each energy bin

   QPen pen;
   pen.setColor(QColor("black"));
   pen.setWidth(iPlotLineWidth);
   painter.setPen(pen);

   bool doOnceOnly = false;

   int iy_min = 0, iy_max = n;

   for (int i=0; i<n; i++)
   {
      if (yAxisMin >= SafeLog(*(lower + i)) && yAxisMin < SafeLog(*(upper + i))) iy_max = i;
      if (yAxisMax >= SafeLog(*(lower + i)) && yAxisMax < SafeLog(*(upper + i))) iy_min = i;
   }

   double subPanelHeight = myPlotHeight/((double)(iy_max - iy_min)), place = myStartY;

   for (int i=iy_min; i<iy_max; i++)
   {

      // energy bin limits lines

      double yValue = SafeLog(*(lower + i));
      energyLowerSauvaud[i] = place;
      double yposLower = energyLowerSauvaud[i];

      yValue = SafeLog(*(upper + i));
      energyUpperSauvaud[i] = place + subPanelHeight;
      double yposUpper = energyUpperSauvaud[i] ;

      place += subPanelHeight;
      
      // centre value
      
      energyCentreValueSauvaud[i] = *(centre + i);

      // energy values

      double yposLabel = (energyLowerSauvaud[i] + energyUpperSauvaud[i])/2.0;
      if (yposLabel > myStartY && yposLabel < myStartY + myPlotHeight)
      {
	 QString yLabel;
	 if (*(centre + i) >= 100) yLabel = QString("%1").arg(*(centre + i), 0, 'e', 1);
         else yLabel = QString("%1").arg(*(centre + i), 0, 'f', 1);
         painter.translate(QPointF(myStartX-gapBetweenAxisAndTick, yposLabel));
         drawRichText(yLabel, fontTicks, ALIGN_RIGHT, &painter);
         painter.resetMatrix();

         if (!doOnceOnly)
	 {
            painter.translate(QPointF(myStartX + myPlotWidth + gapBetweenAxisAndTick*2.2, yposUpper));
            painter.rotate(270);
            drawRichText("180", fontSauv, ALIGN_CENTRE, &painter);
            painter.resetMatrix();
            painter.translate(QPointF(myStartX + myPlotWidth + gapBetweenAxisAndTick*2.2, yposLower));
            painter.rotate(270);
            drawRichText("0", fontSauv, ALIGN_CENTRE, &painter);
            painter.resetMatrix();
            painter.translate(QPointF(myStartX + myPlotWidth + gapBetweenAxisAndTick*2.2, yposLower + 40.));
            painter.rotate(270);
            drawRichText("Flow", fontSauv, ALIGN_RIGHT, &painter);
            painter.resetMatrix();
            doOnceOnly = true;
         }

      }
   }
}

//-------------------------------------------------------------------------------------------------
//
// draw a Sauvaud plot slice
//
//-------------------------------------------------------------------------------------------------

void QsiPlot::drawSauvaudSlice(int iRow, int iCol, double z,
                               double xBegin, double xEnd,
                               double yBegin, double yEnd, double ePos,
                               double colourBarMin, double colourBarMax)
{

   int pos = -1;
   for (int ixs=0; ixs<numEnergyBinSauvaud; ixs++)
   {
//      if (ePos == energyCentreValueSauvaud[ixs]) pos = ixs;
      if (fabs(ePos - energyCentreValueSauvaud[ixs]) < 0.10*energyCentreValueSauvaud[ixs]) pos = ixs;
   }
   
   if (pos == -1) return;

   double myPlotWidth = plotWidths[indexPosition(iRow, iCol)];
   double myPlotHeight = energyUpperSauvaud[pos] - energyLowerSauvaud[pos];
   double myStartX = plotStartsX[indexPosition(iRow, iCol)];
   double myStartY = energyLowerSauvaud[pos];

   double myPlotHeight_Total = plotHeights[indexPosition(iRow, iCol)];
   double myStartY_Total = plotStartsY[indexPosition(iRow, iCol)];

   if (myStartY < myStartY_Total) return;
   if (myStartY + myPlotHeight > myStartY_Total + myPlotHeight_Total) return;

   double zval = SafeLog(z);

   colourBarMin = SafeLog(colourBarMin);
   colourBarMax = SafeLog(colourBarMax);
   
   double yAxisMin = 0.0;
   double yAxisMax = 180.0;
 
   double xAxisMin = axisLimits[indexPosition(iRow, iCol)][0][0];
   double xAxisMax = axisLimits[indexPosition(iRow, iCol)][0][1];
   
   //   yBegin = SafeLog(yBegin);
   //   yEnd = SafeLog(yEnd);

   double scaleFactorX = myPlotWidth/(xAxisMax - xAxisMin);
   double scaleFactorY = myPlotHeight/(yAxisMax - yAxisMin);
   
   if (xBegin < xAxisMin) xBegin = xAxisMin;
   if (xBegin > xAxisMax) xBegin = xAxisMax;
   if (xEnd < xAxisMin) xEnd = xAxisMin;
   if (xEnd > xAxisMax) xEnd = xAxisMax;
   
   if (yBegin < yAxisMin) yBegin = yAxisMin;
   if (yBegin > yAxisMax) yBegin = yAxisMax;
   if (yEnd < yAxisMin) yEnd = yAxisMin;
   if (yEnd > yAxisMax) yEnd = yAxisMax;
   
   double x1 = (xBegin - xAxisMin)*scaleFactorX + myStartX;
   double x2 = (xEnd - xAxisMin)*scaleFactorX + myStartX;
   double y1 = myPlotHeight - (yBegin - yAxisMin)*scaleFactorY + myStartY;
   double y2 = myPlotHeight - (yEnd - yAxisMin)*scaleFactorY + myStartY; 

   if (y1 < myStartY_Total) y1 = myStartY_Total;
   if (y1 > myStartY_Total + myPlotHeight_Total) y1 = myStartY_Total + myPlotHeight_Total;

   if (y2 < myStartY_Total) y2 = myStartY_Total;
   if (y2 > myStartY_Total + myPlotHeight_Total) y2 = myStartY_Total + myPlotHeight_Total;

   QPainter painter(plot);   

   zval = (zval < colourBarMin) ? colourBarMin : zval;
   zval = (zval > colourBarMax) ? colourBarMax : zval;
   
   double col = ((zval - (double)colourBarMin)/((double)(colourBarMax - colourBarMin)));
   col = 1.0 - col;

   if (col != 1.0)
   {
      QColor myColor = colourBarDefinition(col);
      painter.setPen(myColor);
      QBrush myBrush(myColor);
      myBrush.setStyle(Qt::SolidPattern);
      painter.setBrush(myBrush);
      painter.drawRect(QRectF(x1, y1, x2-x1, y2-y1));
   }
}

//-------------------------------------------------------------------------------------------------
//
// draw energy lines for Sauvaud plots
//
//-------------------------------------------------------------------------------------------------

void QsiPlot::drawSauvaudEnergyLines(int iRow, int iCol)
{
 
   double myPlotWidth = plotWidths[indexPosition(iRow, iCol)];
   double myPlotHeight = plotHeights[indexPosition(iRow, iCol)];
   double myStartX = plotStartsX[indexPosition(iRow, iCol)];
   double myStartY = plotStartsY[indexPosition(iRow, iCol)];

// setup axis limits

   bool useLog = false;
   if (scaleTypeY[indexPosition(iRow, iCol)] == Log) useLog = true;

   double yAxisMin, yAxisMax;
   if (useLog)
   {
      yAxisMin = SafeLog(axisLimits[indexPosition(iRow, iCol)][1][0]);
      yAxisMax = SafeLog(axisLimits[indexPosition(iRow, iCol)][1][1]);
   }
   else
   {
      yAxisMin = axisLimits[indexPosition(iRow, iCol)][1][0];
      yAxisMax = axisLimits[indexPosition(iRow, iCol)][1][1];
   }

// setup the painter

   QPainter painter(plot);

// draw lines indicating lower & upper energy bin limits; draw y-axis values
// for each energy bin

   QPen pen;
   pen.setColor(QColor("black"));
   pen.setWidth(iPlotLineWidth);
   painter.setPen(pen);

   for (int i=0; i<numEnergyBinSauvaud; i++)
   {

      // energy bin limits lines

      double yposLower = energyLowerSauvaud[i];
      energyLowerSauvaud[i] = yposLower;
      if (yposLower > myStartY && yposLower < myStartY + myPlotHeight)
         painter.drawLine(QLineF(myStartX, yposLower, myStartX + myPlotWidth, yposLower));
      double yposUpper = energyUpperSauvaud[i];
      energyUpperSauvaud[i] = yposUpper;
      if (yposUpper > myStartY && yposUpper < myStartY + myPlotHeight)
         painter.drawLine(QLineF(myStartX, yposUpper, myStartX + myPlotWidth, yposUpper));
   }
}

//-------------------------------------------------------------------------------------------------
//    
// convert value between 0 and 1 to a colour for spectrograms
//
//-------------------------------------------------------------------------------------------------

QColor QsiPlot::colourBarDefinition(double in)
{
   QColor myColor;
   switch(colourBarDef)
   {
      case 1: // CAA
      double ca[10], cb[10], cc[10];
      ca[0] = 0.0; // red
      ca[1] = 0.18; // orange
      ca[2] = 0.24; // yellow
      ca[3] = 0.43; // green
      ca[4] = 0.55; // cyan
      ca[5] = 0.75; // blue
      ca[6] = 0.85; // indigo
      ca[7] = 0.95; // violet
      ca[8] = 0.98; // violet (black)
      ca[9] = 1.00; // black
      for (int i=0; i<10; i++) cb[i] = 1.0;
      cb[7] = 0.3; cb[8] = 0.1; cb[9] = 0.0;

      cc[0] = 0.0; // red
      cc[1] = 0.1; // orange
      cc[2] = 0.18; // yellow
      cc[3] = 0.26; // green
      cc[4] = 0.48; // cyan
      cc[5] = 0.64; // blue
      cc[6] = 0.72; // indigio
      cc[7] = 0.78; // violet
      cc[8] = 0.85; // violet (black)
      cc[9] = 0.85; // black

      for (int i=0; i<9; i++)
      {
         if (in >= ca[i] && in < ca[i+1])
         {
            double u = cc[i] + (cc[i+1]-cc[i])*(in-ca[i])/(ca[i+1]-ca[i]);
            double uu = cb[i] + (cb[i+1]-cb[i])*(in-ca[i])/(ca[i+1]-ca[i]);
            myColor.setHsvF(u, 1.0, uu);
         }
      }
      if (in == 1.0) myColor.setHsvF(0.0, 0.0, 1.0);
      break;

      case 2: // similar to IDL red-blue
      {
          double red[5][2], green[6][2], blue[5][2];

          red[0][0] = 0.0;  red[0][1] = 0.0;
          red[1][0] = 0.35; red[1][1] = 0.0;
          red[2][0] = 0.66; red[2][1] = 1.0;
          red[3][0] = 0.89; red[3][1] = 1.0;
          red[4][0] = 1.0;  red[4][1] = 0.5;

          green[0][0] = 0.0;   green[0][1] = 0.0;
          green[1][0] = 0.125; green[1][1] = 0.0;
          green[2][0] = 0.375; green[2][1] = 1.0;
          green[3][0] = 0.64;  green[3][1] = 1.0;
          green[4][0] = 0.91;  green[4][1] = 0.0;
          green[5][0] = 1.0;   green[5][1] = 0.0;

          blue[0][0] = 0.0;  blue[0][1] = 0.5;
          blue[1][0] = 0.11; blue[1][1] = 1.0;
          blue[2][0] = 0.34; blue[2][1] = 1.0;
          blue[3][0] = 0.65; blue[3][1] = 0.0;
          blue[4][0] = 1.0;  blue[4][1] = 0.0;

          double uR = 0., uG = 0., uB = 0.;

          in = 1.0 - in;
  
          for (int i=0; i<4; ++i)
          {
             if (in >= red[i][0] && in < red[i+1][0])
             {
                uR = red[i][1] + (red[i+1][1]-red[i][1])*(in-red[i][0])/(red[i+1][0]-red[i][0]);
             }
          }
          if (in <= red[0][0]) uR = red[0][1];
          if (in >= red[4][0]) uR = red[4][1];

          for (int i=0; i<5; ++i)
          { 
             if (in >= green[i][0] && in < green[i+1][0])
             {
                uG = green[i][1] + (green[i+1][1]-green[i][1])*(in-green[i][0])/(green[i+1][0]-green[i][0]);
             }
          }
          if (in <= green[0][0]) uG = green[0][1];
          if (in >= green[5][0]) uG = green[5][1];

          for (int i=0; i<4; ++i)
          { 
             if (in >= blue[i][0] && in < blue[i+1][0])
             {
                uB = blue[i][1] + (blue[i+1][1]-blue[i][1])*(in-blue[i][0])/(blue[i+1][0]-blue[i][0]);
             }
          }
          if (in <= blue[0][0]) uB = blue[0][1];
          if (in >= blue[4][0]) uB = blue[4][1];

          myColor.setRgbF(uR, uG, uB);
  
      }
      break;

      default:
      if (in == 1.0) myColor.setHsvF(0.0, 0.0, 1.0);
      else myColor.setHsvF(in*0.85, 1.0, 1.0);
      break;
   }
   return(myColor);
}

//-------------------------------------------------------------------------------------------------
//
// set colour table
//
//-------------------------------------------------------------------------------------------------

void QsiPlot::setColourBarDefinition(int in)
{
   colourBarDef = in;
}

//-------------------------------------------------------------------------------------------------
//
// determine position in array based on row and column
//
//-------------------------------------------------------------------------------------------------

int QsiPlot::indexPosition(int iRow, int iCol)
{
   int iReturn;
   if (iCol == -1) iReturn = iRow;
   else iReturn = iCol*numRows + iRow;   
   return(iReturn);
}

//-------------------------------------------------------------------------------------------------
//
// set plot type
//
//-------------------------------------------------------------------------------------------------

int QsiPlot::indexPositionReturn(int iRow, int iCol, int iWhat)
{
   int iReturn;
   if (iCol == -1)
   {
      int myCol = iRow/numRows;
      int myRow = iRow%numRows;
      if (iWhat == QS_COL) iReturn = myCol;
      else iReturn = myRow;
   }
   else
   {
      if (iWhat == QS_COL) iReturn = iCol;
      else iReturn = iRow;
   }
   return(iReturn);
}
