/*******************************************************************************
 **
 **  $Id: readcdf.h 165 2008-12-05 10:18:33Z AndrewLahiff $
 **
 ****************************************************************************** 
 ** 
 ** Copyright (C) 2007-2008 Andrew Lahiff.
 ** 
 ** This file is part of ReadFile.
 **
 ** ReadFile is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 ** 
 ** ReadFile is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 ** 
 ** You should have received a copy of the GNU General Public License
 ** along with ReadFile.  If not, see <http://www.gnu.org/licenses/>.
 ** 
 *******************************************************************************/

#ifndef READCDF_H_
#define READCDF_H_

#include<vector>

#include "metadata.h"
#include "abstractreader.h"
#include "cdf.h"
#include "readfile_defs.h"
#include <QFile>
#include <QString>
#include <QTextStream>

#define _FILL_VALUE_INT -123456789

#define _PLUS 1
#define _MINUS -1

class ReadCdf : public AbstractReader
{
   public:
      ReadCdf();
      int open(QString);
      int readHeader();
      int getNumVariables();
      QString getFileBeginDateTime();
      QString getFileEndDateTime();
      QString getVariableAttr(int, QString);
      QString getGlobalAttr(QString);
      int getVariableDimension(int);
      int getVariableSize(int, int);
      DataType getVariableDataType(int);

      int readRecord(bool);
      void close();
      int getDataValue(int, int *);
      int getDataValue(int, short int *);
      int getDataValue(int, float *);
      int getDataValue(int, double *);
      int getDataValue(int, QString *);
      int getEpochValue(int, QString *);
      int getDataValueDelta(int, int, float *);
      int getVariableNumber(QString);
      void rewindData();

   private:
      QString fileName;
      CDFid id;
      CDFstatus status;
      int numVariables, maxRecordsZ, iCurrentDataRecord;
      QString timeFileBegin;
      QString timeFileEnd;
      std::vector<VariableMetaData> varMetaData;
      std::vector<DataType> varDataType;
      std::vector<std::vector<int> > varDataSizes;
      GlobalMetaData varGlobalData;

      QString getGlobalMetaData(char*);
      QString getVariableMetaData(int, char*);
};

#endif /*READCDF_H_*/
